<?php
//   Backlink counter
//   (c) 2014-2015 Scriptol.com
//
//   Require one or more CSV file from Google Webmaster Tools.
//   The content must be a list of comma separated links each followed by two values.
//
//   This scripts build the list of your HTML or PHP pages from a local copy,
//   get the number of backlinks from the CSV file,
//   and build a table into an HTML file  to display the number of backlinks
//   for each of your pages.
//
//   If a page is not in the CSV file, the number of BL is assumed to be zero.
//
//   If several CSV files are provided, the difference between the first value
//   and the last value is displayed.
include_once("path.php");
// Change flags below to filter files
$ignoreImage=true;
$pagesOnly=true;
$minSize=1024;
// End of flags section
$csv=array();
$cellnum=0;
$fh=0;
$root="";
$prefix="";
$rootlen=0;
$ininame="";
$siteurl="";
$sitecount="";
$filecount=0;
$dircount=0;
$sums=array();
// Change the lists below according to your site
$filesToExclude=array("index.php","header.php","top.php","sidebar.php","footer.php","bottom.php");
$extToExclude=array(".zip",".gz",".exe",".xml",".css",".js",".txt",".lib");
$imgExtensions=array(".jpg",".png",".gif",".jpeg",".bmp",".ico");
// Loading the ini file
function loadIni($name)
{
   $ini=array();
   if(!file_exists($name))
   {
      $oldname=$name;
      $name.=".ini";
      if(!file_exists($name))
      {
         die("Ini file '$oldname' or $name' not found.");
      }
   }
   $ini=file($name);
   foreach($ini as $line)
   {
      $line=trim($line);
      if(trim($line)==="")
      {
         continue;
      }
      if(substr($line,0,4)==="site")
      {
         global $root;
         $root=substr($line,5);
         continue;
      }
      if(substr($line,0,6)==="prefix")
      {
         global $prefix;
         $prefix=substr($line,7);
         continue;
      }
      if(substr($line,0,3)==="csv")
      {
         global $csv;
         array_push($csv,substr($line,4));
      }
   }
   return;
}

// Put the csv in order by date.
$sheets=array();
// This CVS line split function is specialized to our problem
// Replace commas in quotes by dots, could be restored to commas at display
function splitCSVLine($iline)
{
   $l=strlen($iline);
   $inquote=false;
   for($i=0;$i<$l;$i++)
   {
      if(($iline{$i}==='"')||($iline{$i}==="'"))
      {
         $inquote=!$inquote;
      }
      if($iline{$i}===",")
      {
         if($inquote)
         {
            $iline{$i}='.';
         }
      }
   }
   return explode(",",$iline);
}

//  Load a CSV file, convert it into array, add to another array of arrays
function loadCSV()
{
   global $csv;
   foreach($csv as $csvname)
   {
      $rawcsv=array();
      if(!file_exists($csvname))
      {
         die("Error, $csvname not found!");
      }
      $rawcsv=file($csvname);
      $urlist=array();
      foreach($rawcsv as $row)
      {
         $rowcells=splitCSVLine($row);
         $url=$rowcells[0];
         $url=trim($url);
         if(substr($url,0,4)==='http')
         {
            global $siteurl;
            $siteurl=$rowcells[0];
            global $sitecount;
            if(strlen($sitecount)>0)
            {
               $sitecount.=" ";
            }
            $sitecount.=strval($rowcells[2]);
            continue;
         }
         if($url{0}==='/')
         {
            $val=$rowcells[2];
            $urlist[$url]=$val;
         }
      }
      asort($urlist);
      global $sheets;
      array_push($sheets,$urlist);
   }
   return;
}

// Make a list of CSV files from the prefix
function getCSVFiles($p,$currdir)
{
   global $csv;
   $csv=array();
   $filelist=scandir($currdir);
   $plen=strlen($p);
   $dummy="";
   $ext="";
   foreach($filelist as $filename)
   {
      if($filename==="")
      {
         break;
      }
      if($filename{0}===".")
      {
         continue;
      }
      global $filesToExclude;
      if(in_array($filename,$filesToExclude))
      {
         continue;
      }
      if(Path::type($filename)==="dir")
      {
         continue;
      }
      if(substr($filename,0,$plen)===$p)
      {
         $_I1=Path::splitExt($filename);
         $dummy=reset($_I1);
         $ext=next($_I1);
         if((strtolower($ext)!="csv"))
         {
            continue;
         }
         array_push($csv,$filename);
      }
   }
   return;
}

// Filter according to some extensions
function extFilter($url)
{
   $ext=Path::getExtension($url);
   global $ignoreImage;
   global $imgExtensions;
   if($ignoreImage&&(in_array(strtolower($ext),$imgExtensions)))
   {
      return true;
   }
   global $pagesOnly;
   if($pagesOnly)
   {
      global $extToExclude;
      if(in_array(strtolower($ext),$extToExclude))
      {
         return true;
      }
   }
   return false;
}

// Get number of backlinks for an URL, from a CSV
function getCount($sheet,$url)
{
   global $sheets;
   $s=$sheets[$sheet];
   if(!array_key_exists($url,$s))
   {
      return 0;
   }
   return intVal($s[$url]);
}

// Add a line in the HTML file
function tohtml($str)
{
   global $fh;
   fwrite($fh,$str);
   return;
}

// Add entry
function addEntry($fullpath,$isdir)
{
   global $rootlen;
   $relpath=substr($fullpath,$rootlen);
   if($isdir)
   {
      tohtml("<tr class='dirhead'>");
      tohtml("<td><b><u>$relpath</u></b></td>");
   }
   else
   {
      tohtml("<tr>");
      tohtml("<td>$relpath</td>");
   }
   $first=0;
   $last=0;
   global $cellnum;
   for($cn=0;$cn<$cellnum;$cn++)
   {
      $backlinks=getCount($cn,$relpath);
      if($cn===0)
      {
         $first=$backlinks;
      }
      if($cn===$cellnum-1)
      {
         $last=$backlinks;
      }
      tohtml("<td>$backlinks</td>");
      global $sums;
      $sums[$cn]+=$backlinks;
   }
   global $cellnum;
   if($cellnum>1)
   {
      $diff=$last-$first;
      $pad="";
      
      if($diff>0)
      {
         if($diff<10)
         {
            $pad="&nbsp;";
         }
         tohtml("<td><b>$pad$diff</b></td>");
      }
      else
      {
         if($diff<0)
         {
            tohtml("<td>$diff</td>");
         }
      else
      {
         tohtml("<td>&nbsp;</td>");
      }
      }
   }
   tohtml("</tr>\n");
   return;
}

// Read the content of a directory, recursively
function readFolder($currdir)
{
   if(!file_exists($currdir))
   {
      echo "Subdirectory '$currdir' not found.", "\n";
      exit();
   }
   if(Path::type($currdir)!="dir")
   {
      die($currdir." not a directory");
   }
   $filelist=scandir($currdir);
   $dirlist=array();
   foreach($filelist as $filename)
   {
      if($filename==="")
      {
         break;
      }
      if($filename{0}===".")
      {
         continue;
      }
      global $filesToExclude;
      if(in_array($filename,$filesToExclude))
      {
         continue;
      }
      $fullpath=Path::merge($currdir,$filename);
      
      $_I1=Path::type($fullpath);
      if($_I1==="dir")
      {
         array_push($dirlist,$fullpath."/");
      }
      else
      {
         if($_I1==="file")
         {
            global $minSize;
            if(filesize($fullpath)<$minSize)
            {
               continue;
            }
            if(extFilter($filename))
            {
               continue;
            }
            addEntry($fullpath,false);
            global $filecount;
            $filecount+=1;
         }
      else
      {
         tohtml("<unknow type> $fullpath");
      }
      }
   }
   foreach($dirlist as $dirpath)
   {
      if(trim($dirpath)==="")
      {
         continue;
      }
      if($dirpath{0}===".")
      {
         continue;
      }
      addEntry($dirpath,true);
      global $dircount;
      $dircount+=1;
      readFolder($dirpath);
   }
   return;
}

echo "Type the name of an ini file or press Enter : ";
$fp=fopen("php://stdin","r");
$ininame=rtrim(fgets($fp,65536));
fclose($fp);
if($ininame==="")
{
   $ininame="backcount.ini";
}

echo "Loading configuration file", " ", $ininame, "\n";
loadIni($ininame);
if(!file_exists($root))
{
   die("Directory '$root' not found.");
}
$rootlen=strlen($root);

echo "Site:", " ", $root, "\n";
if($prefix!="")
{
   echo "CSV prefix:", " ", $prefix, "\n";
   getCSVFiles($prefix,getcwd());
}

$ctr=0;
foreach($csv as $c)
{
   echo $ininame, " ", "CSV:", " ", $c, "\n";
   $sums[$ctr]=0;
   $ctr+=1;
}

$cellnum=count($csv);

if(count($csv)===0)
{
   die("No CSV file found starting with $prefix");
}

loadCSV();
$outname=$csv[count($csv)-1];
$upos=strpos($outname,"_");
$outname=substr($outname,0,$upos+8+1).".html";

$fh=fopen($outname,"w");

tohtml("<!DOCTYPE html\n");
tohtml("<html>\n");
tohtml("<head>\n");
tohtml("<link type='text/css' href='backcount.css' rel='stylesheet'>\n");
tohtml("</head>\n");
tohtml("<body>\n");
tohtml("<h1>$siteurl  $sitecount</h1>\n");
tohtml("<table>\n<tr><td><b><u>Page</u></b>");
for($i=1;$i<=count($csv);$i++)
{
   tohtml("</td><td><b><u>Links</u></b></td>");
}

if(count($csv)>1)
{
   tohtml("<td><b><u>Diff</u></b></td>");
}
tohtml("</tr>\n");
readFolder($root);
tohtml("</table>");
tohtml("</div></body></html>");
tohtml("<p>Total backlinks from different sites:&nbsp;");
for($i=0;$i<$cellnum;$i++)
{
   if($i>0)
   {
      tohtml("then &nbsp;");
   }
   tohtml("<b>".$sums[$i]."</b> &nbsp;");
}

tohtml("</p>");
tohtml("<p>$filecount files. $dircount dirs.</p>");
fclose($fh);

echo "\n";
echo "Total backlinks from different sites: ";
for($i=0;$i<$cellnum;$i++)
{
   echo var_export($sums[$i],true)," ";
}

echo "\n";
echo "$outname created.", "\n";

?>
