
/*
   Backlink counter
   (c) 2014 Scriptol.com
   
   Require one or more CSV file from Google Webmaster Tools.
   The content must be a list of comma separated links each followed by two values.
   
   This scripts get a site map from a standard XML file,
   get the number of backlinks from the CSV file, 
   and build a list into an HTML file  to display the number of backlinks
   for each of your pages.
   
   If a page in the sitemap is not in the CSV file, the number of BL is assumed to be zero.
   
   If several CSV files are provided, the difference between the first value
   and the last value is displayed too.
*/    

include "path.sol"
include "dom.sol"

array csv   ` list of csv filenames   
int cellnum = csv.size()
dir handle
file fh
text sitemap
int rootlen
text ininame
boolean ignoreImage = true
boolean pagesOnly = true
text siteurl          ` Domain 
text sitecount        ` Backlinks to the home page   
int filecount = 0
int dircount = 0

` Change the lists below according to your site

array filesToExclude = { "index.php", "header.php", "top.php", 
      "sidebar.php", "footer.php", "bottom.php" }
      
array extToExclude = {".zip", ".gz", ".exe", ".xml", ".css", ".js", ".txt" }  

array imgExtensions = { ".jpg", ".png", ".gif", ".jpeg", ".bmp", ".ico" }    

` Loading the ini file

void loadIni(text name)
  array ini
  if not Path.exists(name) 
    text oldname = name
    name + ".ini"
    if not Path.exists(name) let die("Ini file '$oldname' or $name' not found.")
  /if  
  ini.load(name)
  for text line in ini
    line = line.trim()
    if line.trim() = "" continue
    if line[0 .. 2] = "map"
      sitemap = line[4 ..]
      continue
    /if
    if line[0 .. 2] = "csv"
      csv.push(line[4 ..])
    /if
  /for
  for text c in csv :  print "CSV:", c ; /for
return        


` Put the csv in order by date. 

array sheets = {}

` This CVS line split function is specialized to our problem
` Replace commas in quotes by dots, could be restored to commas at display

array splitCSVLine(text iline)
  int l = iline.length()
  boolean inquote = false
  for int i in 0 -- l
    if (iline[i] = '"') or (iline[i] = "'")
      inquote = not inquote
    /if
    if iline[i] = "," 
      if inquote ? iline[i] = '.'
    /if 
  /for     
return iline.split(",")

`  Load a CSV file, convert it into array, add to another array of arrays

void loadCSV()

  for text csvname in csv   // load each CSV file from the list at the top
    array rawcsv 
    if not Path.exists(csvname) 
      die("Error, $csvname not found!")
    /if
    rawcsv.load(csvname)
    
    dict urlist    // internal array for a CSV
    for text row in rawcsv    // read each row in the CSV
      array rowcells = splitCSVLine(row)     // split a line
      text url = rowcells[0]
      url = url.trim()
      if url [0 .. 3] = 'http'
        siteurl = url
        rootlen = url.length() - 1
        print "Site url:", siteurl
        sitecount = rowcells[2]
        continue
      /if
      if url[0] = '/'
        text val = rowcells[2]          // only the second value (unique domain) is used
        urlist[url] = val    // add a line in the two-columns array
      /if
    /for
    urlist.sort()
    sheets.push(urlist)   // add a CSV as an array in the array of CSVs   
  /for
return    

` Filter according to some extensions

boolean extFilter(text url)
  text ext = Path.getExtension(url)
  if ignoreImage and (ext in imgExtensions) return true
  if pagesOnly 
    if ext in extToExclude return true
  /if  
return false  


` Get number of backlinks for an URL, from a CSV 

int getCount(int sheet, text url)
  array s = sheets[sheet]
  if not array_key_exists(url, s) return 0
return s[url]  


` Add a line in the HTML file

void tohtml(text str)
  fh.write(str)
return  

` Add entry

void addEntry(text fullpath)
     text relpath = fullpath[rootlen ..]
     if relpath = "" return
     tohtml("<tr>")
     tohtml("<td>$relpath</td>")

     int first
     int last
     for int cn in 0 .. cellnum
        int backlinks = getCount(cn, relpath)
        if cn = 0 ? first = cn
        if cn = cellnum ? last = cn
        tohtml("<td>$backlinks</p>")
     /for
     if cellnum > 1
      int diff = last - first 
      tohtml("<td>$diff</td>")
     /if
      tohtml("</tr>\n")     
return


` Read the content of the XML file

void readMap(text mapname)

   if not Path.exists(mapname)
     print "Sitemap '$mapname' not found."
     exit()
   /if
   
   DOMDocument doc
   DOMNodeList locs
   DOMNode node
   DOMElement url = null
   
   doc.load(mapname)
   
   locs = doc.getElementsByTagName("loc")
   node = locs.item(0)
   
   array filelist
   int i = 0
   
   for i in 0 -- locs.length 
      node = locs.item(i)
      text fname = node.textContent
      print "Node: ", fname
      if fname = "" ? die("empty")
      if fname = nil break
      if extFilter(fname) continue
      addEntry(fname)
      filecount + 1      
   /for   

return

input "Type the name of an ini file or press Enter : ", ininame
if ininame = "" let ininame = "backcount.ini"

print "Loading configuration file", ininame
loadIni(ininame)

print "Loading sitemap ", sitemap

if not Path.exists(sitemap) let die("File '$sitemap' not found.")
rootlen = sitemap.length()

loadCSV()

text outname = csv[ csv.size() - 1]
outname = outname[0 .. 21] + ".html"

fh = fopen(outname, "w")

tohtml("<!DOCTYPE html\n")
tohtml("<html>\n")
tohtml("<head>\n")
tohtml("<link type='text/css' href='backcount.css' rel='stylesheet'>\n")
tohtml("</head>\n")
tohtml("<body>\n")
tohtml("<h1>$siteurl -   $sitecount</h1>\n")

tohtml("<table>\n<tr><td><b><u>Page</u></b>")
for int i in 1 .. csv.size()
  tohtml("</td><td><b><u>Backlinks</u></b></td>")
/for
if csv.size() > 1 ? tohtml("<td>Diff</td>")  
tohtml("</tr>\n");

readMap(sitemap)

tohtml("</table>")    
tohtml("</div></body></html>")    

tohtml("<p>$filecount files.</p>")
tohtml("<p>$dircount dirs.</p>")
fh.close() 

print "Done."  